#!/usr/bin/python
from PyQt4.QtGui import QMainWindow, QAction, QIcon, QWidget, QApplication, qApp, QSizePolicy, QKeySequence, QToolBar 
from PyQt4.QtCore import QUrl, SIGNAL, QTimer, QObject, QT_VERSION_STR, QEvent, Qt
from PyQt4.QtWebKit import QWebView, QWebPage, QWebSettings

import sys
import os
import optparse
from configobj import ConfigObj


if os.path.isfile(os.path.expanduser("~/.wcgbrowser.conf")):
    default_config_file =  os.path.expanduser("~/.wcgbrowser.conf")
elif os.path.isfile("/etc/wcgbrowser.conf"):
    default_config_file = "/etc/wcgbrowser.conf"
else:
    default_config_file = None
    print "No config file found; using defaults."
    
class MainWindow(QMainWindow):
    def createAction(self, text, slot=None, shortcut=None, icon=None, tip=None, checkable=False, signal="triggered()"):
        """Borrowed from 'Rapid GUI Development with PyQT by Mark Summerset'"""
        action = QAction(text, self)
        if icon is not None:
            action .setIcon(QIcon.fromTheme(icon, QIcon(":/%s.png" % icon)))
        if shortcut is not None and not shortcut.isEmpty():
            action.setShortcut(shortcut)
            tip += " (%s)" % shortcut.toString()
        if tip is not None:
            action.setToolTip(tip)
            action.setStatusTip(tip)
        if slot is not None:
            self.connect(action, SIGNAL(signal), slot)
        if checkable:
            action.setCheckable()
        return action

    def onLinkClick(self, url):
        if DEBUG:
            if not url.isValid():
                print "Invalid URL %s" % url
            else:
                print "Load URL %s" %url
                               
    def __init__(self, options, parent = None):
        super(MainWindow, self).__init__(parent)
        #Load config file
        self.options = options
        self.configuration = ConfigObj(self.options.config_file)
        if DEBUG:
            print "loading configuration from '%s'" % options.config_file
            print configuration
        self.build_ui(self.options, self.configuration)

    def build_ui(self, options, configuration):
        self.startUrl = options.url or configuration.get("start_url") or "about:blank"
        inactivity_timeout = options.timeout or int(configuration.get("timeout") or 0)
        self.icon_theme = options.icon_theme or configuration.get("icon_theme") or None
        self.zoomfactor = options.zoomfactor or float(configuration.get("zoom_factor") or 1.0)
        ###Start GUI configuration###
        self.browserWindow = QWebView()
        self.browserWindow.settings().setAttribute(QWebSettings.PrivateBrowsingEnabled, True)
        self.browserWindow.settings().setAttribute(QWebSettings.PluginsEnabled, False)
        self.browserWindow.setZoomFactor(self.zoomfactor)
        #Supposedly this code will make certificates work, but I could never
        #get it to work right.  For now we're just ignoring them.
        ## config = QSslConfiguration.defaultConfiguration()
        ## certs = config.caCertificates()
        ## certs.append(QSslCertificate(QFile("somecert.crt")))
        ## config.setCaCertificates(certs)
      
        if self.icon_theme is not None and QT_VERSION_STR > '4.6':
            QIcon.setThemeName(self.icon_theme)
        self.setCentralWidget(self.browserWindow)
        if DEBUG:
            print options
            print "loading %s" % self.startUrl
        self.browserWindow.load(QUrl(self.startUrl))
        if options.isFullscreen is True:
            self.showFullScreen()
        if options.noNav is not True:
            self.navigationBar = QToolBar("Navigation")
            self.addToolBar(Qt.TopToolBarArea, self.navigationBar)
            self.navigationBar.setMovable(False)
            self.navigationBar.setFloatable(False)
            self.back = self.browserWindow.pageAction(QWebPage.Back)
            self.forward = self.browserWindow.pageAction(QWebPage.Forward)
            self.refresh = self.browserWindow.pageAction(QWebPage.Reload)
            self.stop = self.browserWindow.pageAction(QWebPage.Stop)
            self.quit = self.createAction("I'm &Finished", self.reset_browser, QKeySequence("Alt+F"), None, "Click here when you are done to clear your browsing history and reset the search.")
            self.navigationBar.addAction(self.back)
            self.navigationBar.addAction(self.forward)
            self.navigationBar.addAction(self.refresh)
            self.navigationBar.addAction(self.stop)
            self.navigationBar.addSeparator()
            #Insert bookmarks buttons here.
            self.bookmark_buttons = []
            if configuration.get("bookmarks"):
                for bookmark in configuration.get("bookmarks").iteritems():
                    if DEBUG:
                        print "%s, %s, %s\n" % (bookmark[0], bookmark[1]["url"], bookmark[1]["description"])
                    button = self.createAction(bookmark[0],
                                           lambda url=bookmark[1].get("url"): self.browserWindow.load(QUrl(url)),
                                           QKeySequence.mnemonic(bookmark[0]),
                                           None,
                                           bookmark[1].get("description")
                                           )
                    self.navigationBar.addAction(button)
                self.navigationBar.addSeparator()
            spacer = QWidget()
            spacer.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
            self.navigationBar.addWidget(spacer)
            self.navigationBar.addAction(self.quit)
            
            #This removes the ability to toggle off the navigation bar:
            self.navToggle = self.navigationBar.toggleViewAction()
            self.navToggle.setVisible(False)

            #Call a reset function after timeout
            if inactivity_timeout != 0:
                self.ef = Inactivity_Filter(inactivity_timeout)
                self.installEventFilter(self.ef)
                self.browserWindow.page().installEventFilter(self.ef)
                self.connect(self.ef, SIGNAL("timeout()"), self.reset_browser)

        
        ###CONNECTIONS### 
        self.connect (self.browserWindow, SIGNAL("urlChanged(QUrl)"), self.onLinkClick)
        self.connect (self.browserWindow.page().networkAccessManager(), SIGNAL("sslErrors (QNetworkReply *, const QList<QSslError> &)"), self.sslErrorHandler)

        ###END OF CONSTRUCTOR###

    def reset_browser(self):
        # self.navigationBar.clear() doesn't do its job, so remove the toolbar first, then rebuild the UI.
        self.removeToolBar(self.navigationBar)
        self.build_ui(self.options, self.configuration)


    def sslErrorHandler(self, reply, errorList):
        reply.ignoreSslErrors()
        if DEBUG:
            print ("SSL error ignored")
            for error in errorList:
                print error.errorString()
            
class Inactivity_Filter(QTimer):
    def __init__(self, timeout=0, parent=None):
        super(QTimer, self).__init__(parent)
        self.timeout = timeout
        self.setInterval(self.timeout)
        self.start()
        
    def eventFilter(self, object,  event):
        if event.type() in (QEvent.HoverMove, QEvent.KeyPress, QEvent.KeyRelease, ):
            if DEBUG:
                print ("Activity: %s type %d" % (event, event.type()))
            self.emit(SIGNAL("activity"))
            self.start(self.timeout)
        else:
            if DEBUG:
                print ("Ignored event: %s type %d" % (event, event.type()))
        return QObject.eventFilter(self, object, event)


def main(options, args):
    app = QApplication(args)
    mainwin = MainWindow(options)
    mainwin.show()
    app.exec_()

if __name__ == "__main__":
    parser = optparse.OptionParser()
    parser.add_option("-l", "--url", action="store", type="string", dest="url", help="start browser at URL")
    parser.add_option("-f", "--fullscreen", action="store_true", default=False, dest="isFullscreen", help="start browser FullScreen")
    parser.add_option("-n", "--no-navigation", action="store_true", default=False, dest="noNav", help="start browser without Navigation controls")
    parser.add_option("-c", "--config-file", action="store", default=default_config_file, dest="config_file", help="Specifiy an alternate config file")
    parser.add_option("-d", "--debug", action="store_true", default=False, dest="DEBUG", help="enable debugging output")
    parser.add_option("-t", "--timeout", action="store", type="int",  default=0, dest="timeout", help="define the timeout after which to reset the browser due to user inactivity")
    parser.add_option("-i", "--icon-theme", action="store", default=None, dest="icon_theme", help="override default icon theme with other Qt/KDE icon theme")
    parser.add_option("-z", "--zoom", action="store", type="float", default=0, dest="zoomfactor", help="set the zoom factor for web pages")
    (options, args) = parser.parse_args()
    DEBUG = options.DEBUG
    main(options, args)


